package nl.kallestruik.darena.listeners

import nl.kallestruik.darena.events.CheckpointReachedEvent
import nl.kallestruik.darena.managers.ArenaManager
import nl.kallestruik.darena.types.ArenaFeature
import nl.kallestruik.darena.util.ArenaUtil
import org.bukkit.Bukkit
import org.bukkit.event.EventHandler
import org.bukkit.event.Listener
import org.bukkit.event.player.PlayerMoveEvent

class CheckpointListener(
    private val arenaManager: ArenaManager,
): Listener {
    @EventHandler
    fun onMove(event: PlayerMoveEvent) {
        if (!arenaManager.shouldProcessEvent(ArenaFeature.CHECKPOINTS, event.player))
            return

        val arena = arenaManager.currentArena!!
        val uuid = event.player.uniqueId
        val reachedCheckpoints = arena.session!!.reachedCheckpoints[uuid] ?: mutableSetOf()

        for (checkpoint in arena.session!!.checkpoints) {
            if (reachedCheckpoints.contains(checkpoint.label))
                continue

            if (!checkpoint.isInside(event.to))
                continue

            reachedCheckpoints.add(checkpoint.label)
            arena.session!!.lastCheckpoint[uuid] = checkpoint
            arena.session!!.reachedCheckpoints[uuid] = reachedCheckpoints

            arena.pointsSession!!.giveCheckpointPoints(event.player, checkpoint)

            // Call event to let other handlers do further actions with it.
            Bukkit.getPluginManager().callEvent(CheckpointReachedEvent(event.player, arena, checkpoint))

            if (checkpoint.movePlayer) {
                checkpoint.resolveSpawnRule()
                if (checkpoint.spawnRule != null)
                    checkpoint.spawnRule!!.spawnPlayer(event.player, arena.world)
            }

            if (checkpoint.isFinish)
                ArenaUtil.finishPlayer(event.player, arena, arenaManager)

            break
        }
    }
}