package nl.kallestruik.darena.listeners

import nl.kallestruik.darena.managers.ArenaManager
import nl.kallestruik.darena.managers.ConfigManager
import nl.kallestruik.darena.types.ArenaFeature
import org.bukkit.Material
import org.bukkit.entity.Fireball
import org.bukkit.entity.Player
import org.bukkit.event.EventHandler
import org.bukkit.event.Listener
import org.bukkit.event.block.Action
import org.bukkit.event.entity.EntityDamageByEntityEvent
import org.bukkit.event.entity.EntityExplodeEvent
import org.bukkit.event.player.PlayerInteractEvent

class FireballListener(
    private val arenaManager: ArenaManager,
    private val configManager: ConfigManager,
): Listener {

    @EventHandler
    fun onUse(event: PlayerInteractEvent) {
        if (!arenaManager.shouldProcessEvent(ArenaFeature.ITEM_FIREBALL, event.player))
            return

        if (event.action != Action.RIGHT_CLICK_AIR && event.action != Action.RIGHT_CLICK_BLOCK)
            return

        if (!event.hasItem() || event.item!!.type != Material.FIRE_CHARGE)
            return

        if (event.player.getCooldown(Material.FIRE_CHARGE) > 0) return

        val fireball = event.player.launchProjectile(Fireball::class.java)
        fireball.velocity = fireball.velocity.multiply(3)
        fireball.setIsIncendiary(false)
        fireball.yield = 4f


        event.player.setCooldown(Material.FIRE_CHARGE, configManager.cooldownFireball)

        event.isCancelled = true
    }

    @EventHandler
    fun onEntityExplode(event: EntityExplodeEvent) {
        if (!arenaManager.shouldProcessEvent(ArenaFeature.ITEM_FIREBALL, null))
            return

        for (block in event.blockList()) {
            block.type = Material.AIR
        }
    }

    @EventHandler
    fun onDamage(event: EntityDamageByEntityEvent) {
        if (event.entity !is Player)
            return

        if (!arenaManager.shouldProcessEvent(ArenaFeature.ITEM_FIREBALL, event.entity as Player))
            return

        if (event.damager !is Fireball)
            return

        event.damage = 0.0
    }

    @EventHandler
    fun onAttack(event: EntityDamageByEntityEvent) {
        if (!arenaManager.shouldProcessEvent(ArenaFeature.ITEM_FIREBALL, null))
            return

        if (event.entity !is Fireball)
            return

        event.isCancelled = true
    }

}