package nl.kallestruik.darena.listeners

import net.kyori.adventure.text.Component
import nl.kallestruik.darena.managers.ArenaManager
import nl.kallestruik.darena.types.ArenaFeature
import nl.kallestruik.darena.util.ArenaUtil
import org.bukkit.Bukkit
import org.bukkit.attribute.Attribute
import org.bukkit.entity.Entity
import org.bukkit.entity.Player
import org.bukkit.event.Cancellable
import org.bukkit.event.EventHandler
import org.bukkit.event.Listener
import org.bukkit.event.entity.EntityDamageByEntityEvent
import org.bukkit.event.entity.EntityDamageEvent
import org.bukkit.event.entity.PlayerDeathEvent
import org.bukkit.event.entity.ProjectileHitEvent

class KillListener(
    private val arenaManager: ArenaManager
): Listener {

    @EventHandler
    fun onDeath(event: PlayerDeathEvent) {
        if (!arenaManager.shouldProcessEvent(ArenaFeature.ALWAYS, event.entity))
            return

        event.isCancelled = true
        event.entity.health = event.entity.getAttribute(Attribute.GENERIC_MAX_HEALTH)?.value ?: 20.0

        arenaManager.currentArena!!.session!!.deaths.add(event.entity.uniqueId)

        val killerUUID = arenaManager.currentArena!!.session!!.lastDamage[event.entity.uniqueId]
        val killer = if (killerUUID != null)
            Bukkit.getPlayer(killerUUID)
        else
            null

        if (killer != null)
            arenaManager.currentArena!!.pointsSession!!.giveKillPoints(killer)

        if (arenaManager.shouldProcessEvent(ArenaFeature.DEATH_MESSAGE, event.entity)) {
            if (killer == null)
                arenaManager.currentArena!!.sendMessage(
                    Component.text("${event.entity.name} died")
                )
            else
                arenaManager.currentArena!!.sendMessage(
                    Component.text("${event.entity.name} was killed by ${killer.name}")
                )
        }

        if (arenaManager.shouldProcessEvent(ArenaFeature.RESPAWN, event.entity)) {
            arenaManager.currentArena!!.respawn(event.entity)
        } else {
            if (arenaManager.shouldProcessEvent(ArenaFeature.ALWAYS, event.entity))
                arenaManager.currentArena!!.makeSpectator(event.entity)
        }

        if (arenaManager.currentArena!!.session!!.participants.size <= arenaManager.currentArena!!.config.endConditions.playersLeft) {
            arenaManager.end()
        }

    }

    @EventHandler
    fun onDamage(event: EntityDamageByEntityEvent) {
        checkAndUpdateDamager(event.entity, event.damager, event)
    }

    @EventHandler
    fun onProjectileHit(event: ProjectileHitEvent) {
        checkAndUpdateDamager(event.hitEntity, event.entity, event)
    }

    private fun checkAndUpdateDamager(hit: Entity?, hitter: Entity, event: Cancellable) {
        if (hit !is Player)
            return

        val damager = ArenaUtil.resolveDamager(hitter) ?: return

        if (!arenaManager.shouldProcessEvent(ArenaFeature.PVP, hit)) {
            if (arenaManager.shouldProcessEvent(ArenaFeature.ALWAYS, hit)) {
                if (event is EntityDamageByEntityEvent)
                    if (event.cause != EntityDamageEvent.DamageCause.ENTITY_EXPLOSION
                        && event.cause != EntityDamageEvent.DamageCause.BLOCK_EXPLOSION
                    ) {
                        event.isCancelled = true
                    }
            }
            return
        }

        if (hit.uniqueId == damager.uniqueId)
            return

        arenaManager.currentArena!!.session!!.lastDamage[hit.uniqueId] = damager.uniqueId
    }
}