package nl.kallestruik.darena.types.arena

import com.destroystokyo.paper.Namespaced
import kotlinx.serialization.Serializable
import nl.kallestruik.darena.util.Logger
import nl.kallestruik.dlib.config.annotations.*
import org.bukkit.Material
import org.bukkit.NamespacedKey
import org.bukkit.inventory.ItemStack

@Serializable
data class ArenaItem(
    @Description(["The item type."])
    @DefaultString("AIR")
    val material: Material = Material.AIR,
    @Description(["The amount."])
    @MinInt(1)
    @DefaultInt(1)
    val amount: Int = 1,
    @Description(["The enchantments that will be applied to the item."])
    val enchantments: List<ArenaEnchantment> = listOf(),
    @Description(["Whether the item takes durability damage or not."])
    @DefaultBoolean(false)
    val unbreakable: Boolean = false,
    @Description(["A list of blocks that this block can be placed on. Requires the feature GAMEMODE_ADVENTURE to work.",
        "", "Valid values: all minecraft block ids as seen in game with F3+h on without the minecraft:",
        "For example: stone, granite, andesite, diamond_block"])
    val placeOn: Set<String> = mutableSetOf(),
    @Description(["A list of blocks that this item can be break. Requires the feature GAMEMODE_ADVENTURE to work.",
        "", "Valid values: all minecraft block ids as seen in game with F3+h on without the minecraft:",
        "For example: stone, granite, andesite, diamond_block"])
    val canBreak: Set<String> = mutableSetOf(),
) {
    fun asItemStack(): ItemStack {
        Logger.trace(ArenaItem::class, "asItemStack()")
        val stack = ItemStack(material, amount)
        for (enchantment in enchantments) {
            enchantment.addToItem(stack)
        }

        val meta = stack.itemMeta

        if (placeOn.isNotEmpty()) {
            val placeableKeys = mutableSetOf<Namespaced>()
            for (block in placeOn) {
                //TODO: Make this give a nicer error if it fails.
                placeableKeys.add(NamespacedKey.minecraft(block))
            }

            meta.setPlaceableKeys(placeableKeys)
        }

        if (canBreak.isNotEmpty()) {
            val destroyableKeys = mutableSetOf<Namespaced>()
            for (block in canBreak) {
                destroyableKeys.add(NamespacedKey.minecraft(block))
            }

            meta.setDestroyableKeys(destroyableKeys)
        }

        if (unbreakable) {
            meta.isUnbreakable = true
        }

        stack.itemMeta = meta

        return stack
    }
}
