package nl.kallestruik.darena.types.arena

import kotlinx.serialization.Serializable
import nl.kallestruik.dlib.config.annotations.AllowedKeys
import nl.kallestruik.dlib.config.annotations.AllowedKeysExclusive
import nl.kallestruik.dlib.config.annotations.Description
import org.bukkit.entity.Player
import org.bukkit.inventory.EquipmentSlot

@Serializable
data class ArenaLoadout(
    @Description(["The items that will be in the players hotbar."])
    @AllowedKeys(["0", "1", "2", "3", "4", "5", "6", "7", "8"])
    val hotbar: MutableMap<String, ArenaItem> = mutableMapOf(),
    @Description(["The items that will be in the players armor slots."])
    @AllowedKeysExclusive(["helmet", "head"])
    @AllowedKeysExclusive(["chestplate", "chest", "elytra"])
    @AllowedKeysExclusive(["leggings", "legs", "pants"])
    @AllowedKeysExclusive(["boots", "feet"])
    val armor: MutableMap<String, ArenaItem> = mutableMapOf(),
    @Description(["The item that will be in the players offhand."])
    val offhand: ArenaItem? = null,
    @Description(["The potion effects that will be applied to players upon spawning."])
    val effects: MutableList<ArenaPotionEffect> = mutableListOf(),
) {
    fun equip(player: Player) {
        for (entry in hotbar) {
            val slot = entry.key.toInt()
            player.inventory.setItem(slot, entry.value.asItemStack())
        }

        for (entry in armor) {
            when (entry.key) {
                "helmet", "head" -> player.inventory.setItem(EquipmentSlot.HEAD, entry.value.asItemStack())
                "chestplate", "chest", "elytra" -> player.inventory.setItem(EquipmentSlot.CHEST, entry.value.asItemStack())
                "leggings", "legs", "pants" -> player.inventory.setItem(EquipmentSlot.LEGS, entry.value.asItemStack())
                "boots", "feet" -> player.inventory.setItem(EquipmentSlot.FEET, entry.value.asItemStack())
            }
        }

        if (offhand != null)
            player.inventory.setItemInOffHand(offhand.asItemStack())

        for (effect in effects) {
            effect.apply(player)
        }
    }
}
