package nl.kallestruik.darena.types.arena

import kotlinx.serialization.Serializable
import nl.kallestruik.dlib.config.annotations.Description
import nl.kallestruik.dlib.config.annotations.MaxInt
import nl.kallestruik.dlib.config.annotations.MinInt
import org.bukkit.entity.Player
import org.bukkit.potion.PotionEffect
import org.bukkit.potion.PotionEffectType

@Serializable
data class ArenaPotionEffect(
    @Description(["The name of the potion effect.",
    "", "Valid values: the name of the potion effect as seen in the /effect command without minecraft:",
    "For example: invisibility, strength, haste"])
    val name: String,
    @Description(["The strength of the potion effect."])
    @MinInt(0)
    @MaxInt(255)
    val strength: Int,
    @Description(["The duration of the potion effect in ticks. (20 ticks = 1 second)"])
    @MinInt(0)
    val duration: Int,
    @Description(["Whether the potion effect should be hidden from the player. This hides the particles and the icon."])
    val hidden: Boolean
) {
    fun apply(player: Player) {
        player.addPotionEffect(PotionEffect(PotionEffectType.getByName(name)!!, duration, strength, !hidden, !hidden, !hidden))
    }
}
