package nl.kallestruik.darena.types.arena

import kotlinx.serialization.Serializable
import nl.kallestruik.darena.arenas.world.ArenaWorld
import nl.kallestruik.darena.util.Logger
import nl.kallestruik.dlib.config.annotations.DefaultFloat
import nl.kallestruik.dlib.config.annotations.Description
import nl.kallestruik.dlib.config.annotations.MaxFloat
import nl.kallestruik.dlib.config.annotations.MinFloat
import org.bukkit.Location
import org.bukkit.World
import org.bukkit.entity.Player

@Serializable
data class ArenaSpawn(
    @Description(["The x coordinate of the spawn point."])
    @DefaultFloat(0.0)
    val x: Double,
    @Description(["The y coordinate of the spawn point."])
    @DefaultFloat(0.0)
    val y: Double,
    @Description(["The z coordinate of the spawn point."])
    @DefaultFloat(0.0)
    val z: Double,
    @Description(["The yaw of the spawn point."])
    @MinFloat(-180.0)
    @MaxFloat(180.0)
    @DefaultFloat(0.0)
    val yaw: Float,
    @Description(["The pitch of the spawn point."])
    @MinFloat(-90.0)
    @MaxFloat(90.0)
    @DefaultFloat(0.0)
    val pitch: Float
) {
    fun spawn(world: ArenaWorld, player: Player) {
        Logger.trace(ArenaSpawn::class, "spawn(world: ${world.name}, player: ${player.name})")
        player.teleport(toLocation(world.world))
    }

    fun toLocation(world: World): Location {
        return Location(world, x, y, z, yaw, pitch)
    }
}
