package nl.kallestruik.darena.types.arena

import kotlinx.serialization.SerialName
import kotlinx.serialization.Serializable
import nl.kallestruik.dlib.config.annotations.DefaultBoolean
import nl.kallestruik.dlib.config.annotations.DefaultString
import nl.kallestruik.dlib.config.annotations.Description

@Serializable
class ArenaSpawnRule(
    @SerialName("reuse-spawns")
    @Description(["Whether or not to reuse the spawns for different players."])
    @DefaultBoolean(true)
    val reuseSpawns: Boolean = true,
    @SerialName("reuse-individual")
    @Description(["Whether or not to reuse the spawns for individual players.",
        "When this is set a player will always use the same spawn after spawning there once."])
    @DefaultBoolean(false)
    val reuseIndividual: Boolean = false,
    @Description(["The list of spawns that this spawn rule can pick from.",
    "", "Valid values: any spawn name defined in the spawns section."])
    val spawns: MutableList<String> = mutableListOf(),
    @Description(["The loadout to use for players that are spawned using this spawn rule. Use none to not give them any items.",
    "", "Valid values: any loadout defined in the loadouts section or none."])
    @DefaultString("none")
    val loadout: String = "none"
)
