package nl.kallestruik.darena.types.arena

import nl.kallestruik.darena.arenas.world.ArenaWorld
import nl.kallestruik.darena.util.ArenaUtil
import nl.kallestruik.darena.util.Logger
import org.bukkit.entity.Player
import java.util.*

data class ProcessedArenaSpawnRule(
    val reuseSpawns: Boolean,
    val reuseIndividual: Boolean,
    val spawns: List<ArenaSpawn>,
    val loadout: ArenaLoadout?,
) {
    var lastSpawnIndex = -1
    private val lastForPlayer = mutableMapOf<UUID, Int>()

    constructor(spawnRule: ArenaSpawnRule, spawns: List<ArenaSpawn>, loadout: ArenaLoadout?): this(spawnRule.reuseSpawns, spawnRule.reuseIndividual, spawns, loadout)

    fun spawnPlayers(players: Collection<Player>, world: ArenaWorld) {
        Logger.trace(ProcessedArenaSpawnRule::class, "spawnPlayers(players: $players, world: ${world.name})")
        players.forEach() {player ->
            spawnPlayer(player, world)
        }
    }

    fun spawnPlayer(player: Player, world: ArenaWorld) {
        if (player.health <= 0) {
            player.spigot().respawn()
        }

        Logger.trace(ProcessedArenaSpawnRule::class, "spawnPlayer(player: ${player.name}, world: ${world.name})")
        if (player.uniqueId in lastForPlayer) {
            spawnAndEquip(player, world, spawns[lastForPlayer[player.uniqueId]!!])
            return
        }

        if (!reuseSpawns && lastSpawnIndex + 1 >= spawns.size) {
            Logger.warn(ProcessedArenaSpawnRule::class, "Cant spawn player \"${player.name}\" because there are no spots left and reuse-spawns is set to false.")
            return
        }

        val nextSpawnIndex = (lastSpawnIndex + 1) % spawns.size
        spawnAndEquip(player, world, spawns[nextSpawnIndex])
        lastForPlayer[player.uniqueId] = nextSpawnIndex

        lastSpawnIndex++
    }

    private fun spawnAndEquip(player: Player, world: ArenaWorld, spawn: ArenaSpawn) {
        ArenaUtil.clearPlayer(player)
        spawn.spawn(world, player)
        loadout?.equip(player)
    }
}