package nl.kallestruik.darena.types.countdown

import nl.kallestruik.darena.util.CountdownHelper
import org.bukkit.Bukkit
import org.bukkit.entity.Player
import org.bukkit.plugin.java.JavaPlugin
import org.bukkit.scheduler.BukkitRunnable
import org.bukkit.scheduler.BukkitTask


open class AsyncCountdown(
    val plugin: JavaPlugin,
    val time: Int,
    val subtitle: String,
    val alwaysAsTitle: Boolean = false,
    val alwaysAsActionBar: Boolean = false,
    val after: Runnable,
    ): Countdown {
    var task: BukkitTask? = null

    override fun start(players: Collection<Player>): Countdown {
        var currentTime = time
        task = object : BukkitRunnable() {
            override fun run() {
                if (currentTime > 0) {
                    if (!alwaysAsActionBar && (alwaysAsTitle || currentTime <= 5 || currentTime % 10 == 0)) {
                        CountdownHelper.sendTitleToAll(players, currentTime, subtitle)
                        CountdownHelper.sendActionBarToAll(players, currentTime, subtitle)
                    } else {
                        CountdownHelper.sendActionBarToAll(players, currentTime, subtitle)
                    }

                    currentTime--
                } else {
                    CountdownHelper.clearTitleForAll(players)
                    CountdownHelper.clearActionBarForAll(players)
                    Bukkit.getScheduler().runTask(plugin, after)
                    this@AsyncCountdown.cancel()
                }
            }
        }.runTaskTimerAsynchronously(plugin, 0, 20)

        return this
    }

    override fun cancel() {
        task?.cancel()
    }

}