package nl.kallestruik.darena.util

import nl.kallestruik.darena.exceptions.MaterialNotFoundException
import org.bukkit.Material
import org.bukkit.configuration.ConfigurationSection
import org.bukkit.configuration.file.YamlConfiguration
import java.io.File
import java.io.FileOutputStream
import java.io.IOException

object ConfigHelper {

    fun getOrCreateConfig(file: File, template: String): YamlConfiguration {
        Logger.trace(ConfigHelper::class, "getOrCreateConfig(file: ${file.toString()}, template: $template)")
        if (!file.parentFile.exists())
            file.parentFile.mkdirs()
        if (!file.exists())
            createConfig(file, template)

        val config = YamlConfiguration()
        config.load(file)
        return config
    }

    private fun createConfig(file: File, template: String) {
        Logger.trace(ConfigHelper::class, "createConfig(file: ${file.toString()}, template: $template)")
        try {
            ConfigHelper.javaClass.getResourceAsStream("/$template").use { stream ->
                FileOutputStream(file).use { resStreamOut ->
                    if (stream == null) {
                        Logger.error(ConfigHelper::class, "Cannot get resource \"$template\" from jar file.")
                        throw IOException("Cannot get resource \"$template\" from Jar file.")
                    }
                    var readBytes: Int
                    val buffer = ByteArray(4096)
                    while (stream.read(buffer).also { readBytes = it } > 0) {
                        resStreamOut.write(buffer, 0, readBytes)
                    }
                }
            }
        } catch (e: IOException) {
            e.printStackTrace()
        }
    }

    fun getOrCreateSection(section: ConfigurationSection, key: String): ConfigurationSection {
        Logger.trace(ConfigHelper::class, "getOrCreateSection(section: ${section.currentPath}, key: $key)")
        if (section.contains(key) && section.isConfigurationSection(key)) {
            return section.getConfigurationSection(key)!!
        }

        return section.createSection(key)
    }

    @Throws(MaterialNotFoundException::class)
    fun matchMaterial(material: String): Material {
        Logger.trace(ConfigHelper::class, "matchMaterial(material: $material)")
        return Material.matchMaterial(material) ?: throw MaterialNotFoundException("There is no material with the name '$material'")
    }

    fun parseIntList(string: String): List<Int> {
        Logger.trace(ConfigHelper::class, "parseIntList(string: $string)")
        val list = mutableListOf<Int>()

        for (key in string.split(" ")) {
            list.add(key.toInt())
        }

        return list
    }

    fun intListToString(list: List<Int>): String {
        Logger.trace(ConfigHelper::class, "list: ${list.toString()}")
        val sb = StringBuilder()
        for (i in list) {
            sb.append(i)
        }

        return sb.toString().trim()
    }
}
